/*****************************************************/
/* demowave.c                                        */
/* -- Application demonstrates simple tone           */
/*    generation using MultiMedia wave functions.    */
/* -- To build: "cc -d -DSTRICT demowave.c wave.obj  */
/*               mmsystem.lib demowave.rc"           */
/*****************************************************/
#include <windows.h>
#include <windowsx.h>
#include "demowave.h"
#include "wave.h"

/* Prototypes. */
BOOL CALLBACK __export FDlgProc(HWND hwnd, UINT wm,
  WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK __export LwButtonWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam);

/* Constants. */
#define wmDown   (WM_USER+1000) /* Button went down. */
#define wmUp     (WM_USER+1001) /* Button went up. */
#define cmscPlay 1000           /* Duration of tone. */

/* Globals. */
WNDPROC lpfnButton;
float rghz[] =
    {
    440.00f, 466.16f, 493.88f, 523.25f, 554.37f,
    587.33f, 622.25f, 659.26f, 698.46f, 739.99f,
    783.99f, 830.61f
    };

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lpsz, int wShow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    WNDCLASS wcs;

    GetClassInfo(NULL, "Button", &wcs);
    lpfnButton = wcs.lpfnWndProc;

    if (NULL == hinsPrev)
        {
        wcs.style &= ~CS_DBLCLKS;
        wcs.lpfnWndProc = LwButtonWndProc;
        wcs.hInstance = hins;
        wcs.lpszMenuName = NULL;
        wcs.lpszClassName = szButtonClass;
        if (!RegisterClass(&wcs))
            return 0;
        }

    if (FInitSnd())
        {
        DialogBox(hins, MAKEINTRESOURCE(dlgWave), NULL,
          FDlgProc);
        CloseSnd();
        }

    return 0;
    }

LRESULT CALLBACK __export LwButtonWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam)
/*****************************************************/
/* -- Push button subclass procedure.                */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        break;

    case WM_LBUTTONDOWN:
        SendMessage(GetParent(hwnd), wmDown,
          GetWindowWord(hwnd, GWW_ID), lParam);
        break;

    case WM_LBUTTONUP:
        SendMessage(GetParent(hwnd), wmUp,
          GetWindowWord(hwnd, GWW_ID), lParam);
        break;
        }

    return CallWindowProc(lpfnButton, hwnd, wm, wParam,
      lParam);
    }

BOOL CALLBACK __export FDlgProc(HWND hwnd, UINT wm,
  WPARAM wParam, LPARAM lParam)
/*****************************************************/
/* -- Dialog procedure.                              */
/*****************************************************/
    {
    static UINT csps;   /* Sampling rate (smp/sec). */
    static UINT cbit;   /* Bits per sample. */

    switch (wm)
        {
    default:
        break;

    case WM_INITDIALOG:
        csps = cspsMid;
        CheckRadioButton(hwnd, didLo, didHi, didMid);
        cbit = cbitHi;
        CheckRadioButton(hwnd, did8, did16, did8);
        CheckDlgButton(hwnd, didStereo, FALSE);
        CheckDlgButton(hwnd, didSync, FALSE);
        return TRUE;

    case wmDown:
        FPlaySnd(rghz[wParam - didA],
          IsDlgButtonChecked(hwnd, didSync) ?
            cmscPlay : -1,
          csps, cbit,
          IsDlgButtonChecked(hwnd, didStereo) ? 2 : 1);
        return TRUE;

    case wmUp:
        StopSnd();
        return TRUE;

    case WM_COMMAND:
        switch (wParam)
            {
        default:
            break;

        case IDCANCEL:
        case IDOK:
            EndDialog(hwnd, TRUE);
            return TRUE;

        case didLo:
            csps = cspsLo;
            break;

        case didMid:
            csps = cspsMid;
            break;

        case didHi:
            csps = cspsHi;
            break;

        case did8:
            cbit = cbitHi;
            break;

        case did16:
            cbit = cbitLo;
            break;
            }      /* End switch wParam. */
            break; /* End case WM_COMMAND. */
        }          /* End switch wm. */

    return FALSE;
    }
